% DYNAMIC MONOPOLY PROBLEM--Main Program
% main.m
% Numerical Methods for Macroeconomists, by Jeremy Greenwood and Ricardo 
% Marto, Copywrite, Jeremy Greenwood


%% Housekeeping
clear all % Clear all numbers from previous runs
clc       % Clear screen

%% Parameters
% Declare some global variables that will be passed into the function 
% multshooting.m used for the multiple shooting algorithm and into the
% the function extendpath.m for the extended path algorithm

global d e f g h i ostar ovecms T 

% Set parameters for model

% Demand curve
alpha = 1;   % constant
beta  = 0.5; % slope
% Cost function
gamma = 0.5; % quadratic term
kappa = 0.9; % cost reduction term
% Discount factor
delta = 0.96;
% Time horizon for simulation
T = 10;  % Number of periods--don't set too large

% Compute steady-state level of output

ostar = alpha/( beta + gamma*(1-kappa)*(1-delta*kappa) );

%%  METHOD 1: Solve model taking the Decision-Rule Approach
    
% Set up the Quadratic Formulae for Psi

a = -delta*gamma*kappa;   % Coefficient on squared term
b = gamma + beta + delta*gamma*kappa^2; % Coefficient on linear term
c = -gamma*kappa;                       % Constant term   


p = [a b c];            % Coefficients on quadratic

% Plot the quadratic equation for psi
% Construct grid to plot psi over going from 0 to 1 by increments of 0.1
psigrid = 0:.1:1;       % Grid for psi for plotting
quad4psi = a*psigrid.^2 + b*psigrid + c; % Values of quadratic
figure(1)
plot(psigrid,quad4psi)
title('The quadratic for psi')
xlabel('Psi')
ylabel('Quadratic')

% Find roots of quadratic and take the minimum one. 
psi = min( roots(p) );  % Slope term on law of motion for output
eta = (1-psi)*ostar;    % Constant term on law of motion for output

% Iterate on difference equation for output
% Set up vector to store outputs for decision rule approach

ovec = zeros (T,1);     
time = (0:T-1)';        % Set up vector for time
% Note the the first element in these vectors, ovec(1,1) and time(1,1),
% actually correspond to period zero 

for t = 2:T
    ovec(t,1) = eta + psi*ovec(t-1,1);
    % Starting value for loop is ovec(1,1) = 0 
end
pvec = alpha - beta*ovec/2; % Compute prices over time

% Plot results
% Here a graph with 6 panels is constructed in the form of a 3 by 2 matrix
% The last number refers to the position of the plot in the matrix, 
% running 1 to 6

figure(2)
subplot(3,2,1)                % Multipanel graph--first position 
plot(time,ovec)
title('Output--decision rule approach')
xlabel('Time')
ylabel('Output')

subplot(3,2,2)                % Multipanel graph--second position    
plot(time, pvec)
title('Price--decision rule approach')
xlabel('Time')
ylabel('Price')

%%  METHOD 2: Solve model using multiple shooting

% Set up terms for second-order difference equation
% These are global variables
% Passed into multshooting.m

d = -alpha/(-a);        % Constant term
e = b/(-a);             % Coefficient on lagged output
f = c/(-a);             % Coefficient on lagged output from two periods ago

% Create vector to store outputs using multiple shooting
ovecms = zeros (T,1); % This is a global variable   
o2guess = .5*ovec(2,1);
o2 = fzero(@multshooting, o2guess);
% Check solution from MATLAB
errorms = abs (multshooting(o2) ); % absolute value of error in solution
if errorms >= 1.E-9
    disp('Solution has not been found')
    disp( errorms ) % error
end
% Note that the vector ovecms is passed back from the function 
% multshooting using the global statement. The statement multshooting(o2)
% ensures that this vector is oomputed using MATLAB's final solution
% for o2
pvecms = alpha - beta*ovecms/2; % Compute prices over time

% Plot results

subplot(3,2,3)
plot(time,ovecms)
title('Output--multiple shooting')
xlabel('Time')
ylabel('Output')

subplot(3,2,4)
plot(time, pvecms)
title('Price--multiple shooting')
xlabel('Time')
ylabel('Price')

%% METHOD 3: Solve model using extended-path method

% Set up terms for second-order difference equation
% These are global variables
% Passed into extendedpath.m

g = alpha/b;           % Constant term
h = gamma*kappa/b;     % Coefficient on lagged output
i = -a/b;              % Coefficient on future output

guesspath = 0:ostar/(T-1):ostar;
guesspath = guesspath';
errorep = 1;
iterationep = 1;
while iterationep <= 100 &&  errorep >= 1.E-9
    % CollectDataEp       % This line collects data and can ignored.
    % When turned off also turn off DumpData below.
    ovecep    = extendpath(guesspath);
    errorep   = norm(ovecep-guesspath);
    guesspath = ovecep;
    iterationep = iterationep + 1;
end
% DumpDataEp               % This line dumps data and can be ignored.
disp('Number of iterations for extended path method')
disp(iterationep-1)

pvecep = alpha - beta*ovecep/2; % Compute prices over time


% Plot results

subplot(3,2,5)
plot(time,ovecep)
title('Output--extended path')
xlabel('Time')
ylabel('Output')

subplot(3,2,6)
plot(time, pvecep)
title('Price--extended path')
xlabel('Time')
ylabel('Price')    
    


         

